/* utility.c -- A small set of utility functions I've found useful.
   $Id: utility.c,v 0.4 1999/04/22 09:05:04 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1997, 1999 Board of Trustees, Leland Stanford Jr. University

   This program is free software.  See README included in the distribution
   for terms and conditions.

   Some general utility functions, such as warn(), die(), and xmalloc(),
   that make safe programming, standard exit conditions, and the like a bit
   easier to do. */

#include "config.h"             /* size_t portability */
#include "utility.h"            /* Prototypes */

#include <errno.h>              /* errno */
#include <stdarg.h>             /* va_list, va_start(), va_end() */
#include <stdio.h>              /* vfprintf(), stderr */
#include <stdlib.h>             /* malloc() */
#include <string.h>             /* strerror() */
#include <sys/types.h>          /* size_t */


/***************************************************************************
* Error messages
***************************************************************************/

/* Called for non-fatal error messages, sends error message to stderr. */
void warn (char *format, ...)
{
    va_list message;

    va_start (message, format);
    vfprintf (stderr, format, message);
    va_end (message);
    fprintf (stderr, "\n");
}


/* Same as warn(), but add the results of strerror (errno). */
void syswarn (char *format, ...)
{
    va_list message;

    va_start (message, format);
    vfprintf (stderr, format, message);
    va_end (message);
    fprintf (stderr, ": %s\n", strerror (errno));
}  


/* Fatal error message.  Print error to stderr and exit with status 1. */
void die (char *format, ...)
{
    va_list message;

    va_start (message, format);
    vfprintf (stderr, format, message);
    va_end (message);
    fprintf (stderr, "\n");
    exit (1);
}


/* Same as die(), but add strerror (errno). */
void sysdie (char *format, ...)
{
    va_list message;

    va_start (message, format);
    vfprintf (stderr, format, message);
    va_end (message);
    fprintf (stderr, ": %s\n", strerror (errno));
    exit (1);
}  


/***************************************************************************
* Memory allocation
***************************************************************************/

/* malloc() memory, calling sysdie() on failure. */
void *xmalloc (size_t size)
{
    void *p;

    p = malloc (size);
    if (p == NULL) sysdie ("Unable to allocate memory");
    return p;
}


/* realloc() memory, calling sysdie() on failure. */
void *xrealloc (void *p, size_t size)
{
    p = realloc (p, size);
    if (p == NULL) sysdie ("Unable to allocate memory");
    return p;
}
